package ru.amse.bazylevich.faeditor.fautomaton.algorithms;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.ICondition;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Automaton;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Condition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.State;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Transition;



public class TransformerFAutomaton {

	private static class SetStates {
		private Set<IState> mySetStates;
		private IState myState;
		
		public SetStates(IState state, Set<IState> setStates) {
			myState = new State(state.getLabel());
			mySetStates = new LinkedHashSet<IState>(setStates);
		}
		
		public SetStates(IState state) {
			this(state, new LinkedHashSet<IState>());
			mySetStates.add(state);
		}
	}
	
	private static class SetTransition {
		IState myEndState;
		LinkedHashSet<ITransition> myTransition;
		
		public SetTransition(ITransition transition) {
			myEndState = transition.getEnd();
			myTransition = new LinkedHashSet<ITransition>();
			myTransition.add(transition);
			
		}
	}


	public static IAutomaton toDeterminate(IAutomaton automaton) {
		IAutomaton newAutomaton = new Automaton();
		if (automaton.getInitialState() != null) {
			List<SetStates> list = makeStates(automaton);
			for (SetStates sStates : list) {
				makeState(sStates, automaton, newAutomaton);
			}
			for (IState state : newAutomaton.getStates()) {
				makeTransitions(state);
			}
		}
		return newAutomaton;
	}
	
	
	
	private static List<SetStates> makeStates(IAutomaton oldAutomaton) {
		ArrayList<SetStates> listStates = new ArrayList<SetStates>();
		IState initialState = oldAutomaton.getInitialState();
		SetStates setStates = new SetStates(initialState);

		listStates.add(setStates);
	
		for (int i = 0; i < listStates.size(); i++) {
			SetStates sStates = listStates.get(i);
			for (Character symbol : makeConditions(sStates)) {
				Set<IState> set = determineSetStates(sStates, symbol);
				if (containsInList(set, listStates)) {
					addTransition(sStates.myState, symbol, getSetStates(set, listStates).myState);
				} else {
					SetStates newSetStates = new SetStates(new State(makeLabel(set)), set);
					listStates.add(newSetStates);
					addTransition(sStates.myState, symbol, newSetStates.myState);
				}
			}
		}
		return listStates;
	}
	
	
	private static boolean containsInList(Set<IState> setStates, List<SetStates> list) {
		for (SetStates set : list) {
			if (set.mySetStates.size() == setStates.size() && set.mySetStates.containsAll(setStates)) {
				return true;
			}
		}	return false;
	}
	
	private static SetStates getSetStates(Set<IState> setStates, List<SetStates> list) {
		for (SetStates sStates : list) {
			if ((setStates.size() == sStates.mySetStates.size()) && sStates.mySetStates.containsAll(setStates)) {
				return sStates;
			}
		}
		return null;
	}
	
	private static Set<IState> determineSetStates(SetStates setStates, Character symbol) {
		Set<IState> set = new LinkedHashSet<IState>();
		for (IState state : setStates.mySetStates) {
			for (ITransition transition : state.getTransitions()) {
				if (transition.getCondition().getSymbols().contains(symbol)) {
					set.add(transition.getEnd());
				}
			}
		}
		return set;
	}
	
	
	private static Set<Character> makeConditions(SetStates setStates) {
		Set<Character> conditions = new LinkedHashSet<Character>();
		for (IState state : setStates.mySetStates) {
			for (ITransition transition : state.getTransitions()) {
				conditions.addAll(transition.getCondition().getSymbols());
			}
		}
		return conditions;
	}
	
	private static void makeState(SetStates setStates, IAutomaton oldAutomaton, IAutomaton newAutomaton) {
		if (setStates.mySetStates.size() == 1 && setStates.mySetStates.contains(oldAutomaton.getInitialState())) {
			newAutomaton.setInitialState(setStates.myState);
		} else if (isFinalState(setStates.mySetStates, oldAutomaton)) {
			newAutomaton.addFinalState(setStates.myState);
		} else {
			newAutomaton.addState(setStates.myState);
		}
	}
	
	private static boolean isFinalState(Set<IState> setStates, IAutomaton oldAutomaton) {
		for (IState state : setStates) {
			if (oldAutomaton.getFinalStates().contains(state)) {
				return true;
			}
		}
		return false;
	}
	
	private static String makeLabel(Set<IState> setStates) {
		if (setStates.size() > 0 ) {
			StringBuilder sb = new StringBuilder();
			for (IState state: setStates) {
				sb.append(state.getLabel());
			}
			return sb.toString();
		}
		return null;
	}
	
	
	private static SetTransition containsSetTransition(IState state, List<SetTransition> list) {
		for (SetTransition set : list) {
			if (set.myEndState == state) {
				return set;
			}
		}
		return null;
	}
	
	private static void makeTransitions(IState state) {
		List<SetTransition> setTransition = new ArrayList<SetTransition>();
		for (ITransition transition : state.getTransitions()) {
			SetTransition set = containsSetTransition(transition.getEnd(), setTransition);
			if (set != null) {
				set.myTransition.add(transition);
			} else {
				set = new SetTransition(transition);
				setTransition.add(set);
			}
		}
		if (state.getTransitions().size() != setTransition.size()) {
			for (SetTransition set : setTransition) {
				if (set.myTransition.size() > 1) {
					for (ITransition transition : set.myTransition) {
						state.removeTransition(transition);
					}
					Set<Character> conditions = new LinkedHashSet<Character>();
					for (ITransition transition : set.myTransition) {
						conditions.addAll(transition.getCondition().getSymbols());
					}
					addTransition(state, conditions, set.myEndState);
				}
			}
		}
	
	}
	
	
	private static void addTransition(IState start, Character ch, IState end) {
		ICondition condition = new Condition(ch);
		ITransition transition = new Transition(start, end, condition);
		start.addTransition(transition);
	}
	
	private static void addTransition(IState start, Set<Character> set, IState end) {
		ICondition condition = new Condition(set);
		ITransition transition = new Transition(start, end, condition);
		start.addTransition(transition);
	}
	
	
}
