package ru.amse.bazylevich.faeditor.fautomaton.file;

import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;

public class AutomatonSaver {
	
	public static void saveToFile(XMLStreamWriter writer, IAutomaton automaton) throws XMLStreamException {
		writer.writeStartDocument();
		saveToFileWhithoutTitle(writer, automaton);
		writer.close();
	}
	
	public static void saveToFileWhithoutTitle(XMLStreamWriter writer, IAutomaton automaton) throws XMLStreamException {
		writer.writeStartElement("automaton");
		List<IState> statesList = new ArrayList<IState>();
		statesList.addAll(automaton.getStates());
		writer.writeAttribute("initialstate", (new Integer(statesList.indexOf(automaton.getInitialState()))).toString());
		for (IState state : statesList) {
			writer.writeStartElement("state");
			writer.writeAttribute("label", state.getLabel());
			if (automaton.getFinalStates().contains(state)) {
				writer.writeAttribute("type", "final");
			} 
			writer.writeEndElement();
		}
		for (IState state : statesList) {
			for (ITransition transition: state.getTransitions()) {
				writer.writeStartElement("transition");
				writer.writeAttribute("start", (new Integer(statesList.indexOf(transition.getStart()))).toString());
				writer.writeAttribute("end", (new Integer(statesList.indexOf(transition.getEnd())).toString()));
				writer.writeAttribute("condition", makeConditionString(transition.getCondition().toString()));
				writer.writeEndElement();
			}
		}
		writer.writeEndElement();
	}
	
	private static String makeConditionString(String string) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < string.length(); i++) {
			char ch = string.charAt(i);
			if (ch != '\40' && ch != ',') {
				sb.append(ch);
			}
		}
		return sb.toString();
	}

}
