package ru.amse.bazylevich.faeditor.ui.fautomaton.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IAutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;

public class AutomatonPresentation implements IAutomatonPresentation {
	
	public final double MYSEARCHRADIUS = 10;

	
	private final IAutomaton myAutomaton;
	private final Set<IStatePresentation> myStatesPresentations;
	private final Map<IState, IStatePresentation> myMapStatesPresentations;
	
	public AutomatonPresentation(IAutomaton automaton, Set<IStatePresentation> set) {
		myAutomaton = automaton;
		myStatesPresentations = new LinkedHashSet<IStatePresentation>(set);
		myMapStatesPresentations = new HashMap<IState, IStatePresentation>();
		for (IStatePresentation presentation : set) {
			myMapStatesPresentations.put(presentation.getState(),presentation);
		}
	}
	
	public AutomatonPresentation(IAutomaton automaton) {
		this(automaton, new LinkedHashSet<IStatePresentation>());
	}
	
	public Set<IStatePresentation> getStatesPresentations() {
		return Collections.unmodifiableSet(myStatesPresentations);
	}
	
	public void addStatePresentation(IStatePresentation statePresentation) {
		myStatesPresentations.add(statePresentation);
		myMapStatesPresentations.put(statePresentation.getState(), statePresentation);
	}
	
	public Map<IState, IStatePresentation> getMapStatesPresentation() {
		return Collections.unmodifiableMap(myMapStatesPresentations);
	}
	
	public IAutomaton getAutomaton() {
		return myAutomaton;
	}
	
	@Override
	public void removeStatePresentation(IStatePresentation presentation) {
		myStatesPresentations.remove(presentation);
		myAutomaton.removeState(presentation.getState());
	}
	
}
