package ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl;

import java.awt.Graphics;
import java.awt.event.MouseEvent;

import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.ITool;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.AutomatonPresentatios;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.Point;

public class DefaultTool implements ITool {
	
	private final JFAutomaton myJFAutomaton;
	private ITool myTool;
	
	public DefaultTool(JFAutomaton jFAutomaton) {
		myJFAutomaton = jFAutomaton;
		myTool = new DrawRectangleState(jFAutomaton);
	}
	
	@Override
	public void mouseClicked(MouseEvent mEvent) {
		if (mEvent.getClickCount() == 1) {
			if (myJFAutomaton.getComponentCount() != 0) {
				if (mEvent.getSource() == myJFAutomaton) {
					myJFAutomaton.removeAll();
				}
				myJFAutomaton.getTypeOfStateChooser().setState(null);
			}
		}
		if (mEvent.getClickCount() == 2) {
			myTool = new ChangeLabelState(myJFAutomaton);
		} else if (mEvent.getClickCount() == 1) {
			Point point = new Point(mEvent.getX(), mEvent.getY());
			ITransition transition = AutomatonPresentatios.getTransition(myJFAutomaton.getAutomatonPresentation(), point);
			if (transition != null) {
				myTool = new SelectTransitionState(myJFAutomaton, transition);
			} else {
				myTool = new DrawRectangleState(myJFAutomaton);
			}
		}
		myTool.mouseClicked(mEvent);
		
	
	}
	

	@Override
	public void mousePressed(MouseEvent mEvent) {
		Point point = new Point(mEvent.getX(), mEvent.getY());
		ITransition transition = myJFAutomaton.getSelectedTransition();
		if (transition != null) {
			Boolean isStart = AutomatonPresentatios.isStart(myJFAutomaton.getAutomatonPresentation(), transition, point);
			if (isStart != null) {
				myTool = new MoveTransitionState(myJFAutomaton, isStart);
			} else {
				myTool = new DrawRectangleState(myJFAutomaton);
				myJFAutomaton.fireElementsUnselected();
			}
		} else {
			IStatePresentation state = AutomatonPresentatios.searchStatePresentation(myJFAutomaton.getAutomatonPresentation(), point);
			if (state != null) {
    			myTool = new MoveVertexState(myJFAutomaton);
			} else {
				myTool = new DrawRectangleState(myJFAutomaton);
			}
		}
		myTool.mousePressed(mEvent);
	}
	
	@Override
	public void mouseDragged(MouseEvent mEvent) {
		myTool.mouseDragged(mEvent);
	}
	
	@Override
	public void mouseReleased(MouseEvent mEvent) {
		myTool.mouseReleased(mEvent);
	}
	
	
	public void finishDrawing(Graphics gc) {
		myTool.finishDrawing(gc);
	}
	
	@Override
	public void mouseMoved(MouseEvent event) {
		// TODO Auto-generated method stub
		
	}
}
