package ru.amse.bazylevich.faeditor.ui.fautomaton.tools.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.util.LinkedHashSet;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.ICondition;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.impl.Condition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl.AutomatonDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.util.Geometry;
import ru.amse.bazylevich.faeditor.ui.fautomaton.tools.ITool;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.AutomatonPresentatios;
import ru.amse.bazylevich.faeditor.ui.fautomaton.util.Point;

public class MoveTransitionState implements ITool {

	
	private final JFAutomaton myJFAutomaton;
	private Point myDraggedPoint;
	private Point myEndPoint;
	private Point myStartPoint;
	private final boolean myIsDraggedStart;

	public MoveTransitionState(JFAutomaton jFAutomaton, boolean isDraggedStart) {
		myJFAutomaton = jFAutomaton;
		myIsDraggedStart = isDraggedStart;
	}
	
	@Override
	public void finishDrawing(Graphics gc) {
		ITransition transition =  myJFAutomaton.getSelectedTransition();
		if (transition != null) {
			if (myStartPoint != null && myEndPoint != null) {
				myJFAutomaton.getDraw().drawAddingTransition(transition.getCondition().toString(), myStartPoint, myEndPoint, gc);
			}
			drawSelectionTransition(gc);
		}
	}

	private void drawSelectionTransition(Graphics gc) {
		gc.setColor(Color.GRAY);
		gc.fillRoundRect(myStartPoint.getX() - JFAutomaton.MYWIDTH / 2, myStartPoint.getY() - JFAutomaton.MYWIDTH / 2, JFAutomaton.MYWIDTH, JFAutomaton.MYWIDTH, JFAutomaton.MYWIDTH / 2, JFAutomaton.MYWIDTH /2);
		gc.fillRoundRect(myEndPoint.getX() - JFAutomaton.MYWIDTH / 2, myEndPoint.getY() - JFAutomaton.MYWIDTH / 2, JFAutomaton.MYWIDTH, JFAutomaton.MYWIDTH, JFAutomaton.MYWIDTH / 2 , JFAutomaton.MYWIDTH / 2);
		gc.setColor(Color.BLACK);
	}

	

	@Override
	public void mouseClicked(MouseEvent event) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseDragged(MouseEvent event) {
		myJFAutomaton.fireComponentChanged();
		if (myIsDraggedStart) {
			myStartPoint.setX(myStartPoint.getX() + event.getX() - myDraggedPoint.getX());
			myStartPoint.setY(myStartPoint.getY() + event.getY() - myDraggedPoint.getY());
		} else {
			myEndPoint.setX(myEndPoint.getX() + event.getX() - myDraggedPoint.getX());
			myEndPoint.setY(myEndPoint.getY() + event.getY() - myDraggedPoint.getY());
		}
		myDraggedPoint = new Point(event.getX(), event.getY());
		myJFAutomaton.repaint();
	}

	@Override
	public void mousePressed(MouseEvent event) {
		Point point = new Point(event.getX(), event.getY());
		ITransition selectedTransition = myJFAutomaton.getSelectedTransition();
		myDraggedPoint = new Point(point);
		IStatePresentation start = myJFAutomaton.getAutomatonPresentation().getMapStatesPresentation().get(selectedTransition.getStart());
		IStatePresentation end = myJFAutomaton.getAutomatonPresentation().getMapStatesPresentation().get(selectedTransition.getEnd());
		Point startPoint = new Point(start.getX(), start.getY());
		if (start != end) {
			Point endPoint = new Point(end.getX(), end.getY());
			myStartPoint = Geometry.computingEdgeEnd(startPoint, endPoint);
			myEndPoint = Geometry.computingEdgeEnd(endPoint, startPoint);
		} else {
			myStartPoint = new Point(startPoint.getX(), startPoint.getY() - AutomatonDrawer.MYSTATEHEIGHT /2);
			myEndPoint = new Point(myStartPoint);
		}
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		Point point = new Point(event.getX(), event.getY());
		IStatePresentation newEnd = AutomatonPresentatios.getNearStatePresentation(myJFAutomaton.getAutomatonPresentation(), point);
		ITransition transition = myJFAutomaton.getSelectedTransition();
		if (newEnd != null) {
			myJFAutomaton.getAutomatonPresentation().getAutomaton().removeTransition(transition);
			ITransition duplication = null;
			if (myIsDraggedStart) {
				for (ITransition oldTransition : newEnd.getState().getTransitions()) {
					if (oldTransition.getStart() == newEnd.getState()) {
						duplication = oldTransition;
					}
				}
				if (!processDuplication(duplication)) {
					newEnd.getState().addTransition(transition);
					transition.setStart(newEnd.getState());
				}
			} else {
				for (ITransition oldTransition : transition.getStart().getTransitions()) {
					if (oldTransition.getEnd() == newEnd.getState()) {
						duplication = oldTransition;
					}
				}
				if (!processDuplication(duplication)) {
					transition.getStart().addTransition(transition);
					transition.setEnd(newEnd.getState());
				}
			}
				
		}
		myJFAutomaton.setSelectedTransition(null);
		myDraggedPoint = null;
		myStartPoint = null;
		myEndPoint = null;
		myJFAutomaton.repaint();
	}

	private boolean processDuplication(ITransition duplication) {
		if (duplication != null) {
			Set<Character> newCond = new LinkedHashSet<Character>(duplication.getCondition().getSymbols());
			newCond.addAll(myJFAutomaton.getSelectedTransition().getCondition().getSymbols());
			ICondition newCondition = new Condition(newCond);
			duplication.setCondition(newCondition);
			return true;
		}
		return false;
	}
	
	@Override
	public void mouseMoved(MouseEvent event) {
		// TODO Auto-generated method stub
		
	}
}
