package ru.amse.bazylevich.faeditor.ui.fautomaton.util;

import java.util.LinkedHashSet;
import java.util.Set;

import ru.amse.bazylevich.faeditor.fautomaton.IAutomaton;
import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IAutomatonPresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl.AutomatonDrawer;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.util.Geometry;
import ru.amse.bazylevich.faeditor.ui.fautomaton.impl.StatePresentation;


public class AutomatonPresentatios {
	
	public static IStatePresentation searchStatePresentation(IAutomatonPresentation aPresentation, Point point) {
		for (IStatePresentation state : aPresentation.getStatesPresentations()) {
			Point statePoint = new Point(state.getX(), state.getY());
			if (isInEllipse(point, statePoint, AutomatonDrawer.MYSTATEHEIGHT, AutomatonDrawer.MYSTATEWIDTH)) {
				return state;
			}
		}
		return null;
	}
	
	private static boolean isInEllipse(Point point, Point statePoint, int heigth, int width) {
		int x1 = statePoint.getX();
		int y1 = statePoint.getY();
		int x = point.getX();
		int y = point.getY();
		double heightPowTwo = Math.pow(heigth / 2, 2);
		double widthPowTwo = Math.pow(width / 2, 2);

		return ((Math.pow(x1 - x, 2) / widthPowTwo + Math.pow(y1 - y, 2) / heightPowTwo) <= 1);
	}
	
	
	public static Set<IStatePresentation> getStateInRectangle(IAutomatonPresentation aPresentation,Point start, Point end) {
		Set<IStatePresentation> set = new LinkedHashSet<IStatePresentation>();
		for (IStatePresentation sPresentation : aPresentation.getStatesPresentations()) {
			if (isInRectangle(sPresentation, start, end)) {
				set.add(sPresentation);
			}
		}
		return set;
	}
	
	private static boolean isInRectangle(IStatePresentation sPresentation,Point start, Point end) {
		Point rigth = new Point(sPresentation.getX() - AutomatonDrawer.MYSTATEWIDTH / 2, sPresentation.getY() - AutomatonDrawer.MYSTATEHEIGHT /2);
		Point left = new Point(sPresentation.getX() + AutomatonDrawer.MYSTATEWIDTH / 2, sPresentation.getY() + AutomatonDrawer.MYSTATEHEIGHT /2);
		return isBetweenPoints(start, rigth, end) && isBetweenPoints(start, left, end);
	}
	
	
	public static ITransition getTransition(IAutomatonPresentation aPresentation, Point point) {
		for (IStatePresentation sPresentation: aPresentation.getStatesPresentations()) {
			Point start = new Point(sPresentation.getX(), sPresentation.getY());
			for (ITransition transition : sPresentation.getState().getTransitions()) {
				IStatePresentation endStatePresentation = getStatePresentation(aPresentation, transition.getEnd());
				if (endStatePresentation != null) {
					if (endStatePresentation != sPresentation) {
						Point end = new Point(endStatePresentation.getX(), endStatePresentation.getY());
						Point newStart = Geometry.computingEdgeEnd(start, end);
						Point newEnd = Geometry.computingEdgeEnd(end, start);
						if (isOnLine(newStart, newEnd, point) && isBetweenPoints(newStart, point, newEnd)) {
							return transition;
						}
					} else {
						Point center = new Point(start.getX(), start.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2 - AutomatonDrawer.MYLOOPHEIGHT / 2);
						Point end = new Point(point.getX() - center.getX(), point.getY() - center.getY());
						int sum = (int)(Math.pow(end.getX(), 2) + Math.pow(end.getY(), 2));
						int broad = AutomatonDrawer.MYLOOPHEIGHT * AutomatonDrawer.MYLOOPHEIGHT / 4;
						if (sum > broad - 150 && sum < broad + 150) {
							return transition;
						}
					}
				}
			}
			
		}
		return null;
	}
	
	private static boolean isOnLine(Point start, Point end, Point point) {
		int x1 = point.getX() - start.getX();
		int y1 = point.getY() - start.getY();
		
		int x2 = end.getX() - start.getX();
		int y2 = end.getY() - start.getY();
		
		int mult = x1 * y2 - y1 * x2; 
		double length = Math.pow(x2 * x2 + y2 * y2, 0.5);
		
		return Math.abs(mult/ length) < 10;
	}
	
	private static boolean isBetweenPoints(Point start, Point point, Point end) {
		if (start.getX() == end.getX()) {
			return isNearX(start.getX(), point.getX()) && isBetweenPoints(start.getY(), point.getY(), end.getY());
		} else if (start.getY() == end.getY()) {
			return isNearX(start.getY(), point.getY()) && isBetweenPoints(start.getX(), point.getX(), end.getX());
		}
		return isBetweenPoints(start.getX(), point.getX(), end.getX()) && 
				isBetweenPoints(start.getY(), point.getY(), end.getY());
	}
	
	private static boolean isNearX(int x, int x2) {
		return (Math.abs(x - x2) < 10);
	}
	
		
	private static boolean isBetweenPoints(int x1, int x, int x2) {
		return (x >= Math.min(x1, x2)) && (x <= Math.max(x1, x2));
	}
	
	public static IStatePresentation getStatePresentation(IAutomatonPresentation aPresentation, IState state) {
		return aPresentation.getMapStatesPresentation().get(state);
	}
	
	
	public static Boolean isStart(IAutomatonPresentation aPresentation, ITransition transition, Point point) {
		IStatePresentation start = aPresentation.getMapStatesPresentation().get(transition.getStart());
		IStatePresentation end = aPresentation.getMapStatesPresentation().get(transition.getEnd());
		if (end != start) {
			Point startPoint = Geometry.computingEdgeEnd(new Point(start.getX(), start.getY()), new Point(end.getX(), end.getY()));
			Point endPoint = Geometry.computingEdgeEnd(new Point(end.getX(), end.getY()), new Point(start.getX(), start.getY()));
			Point startVertex1 = new Point(startPoint.getX() - JFAutomaton.MYWIDTH / 2, startPoint.getY() - JFAutomaton.MYWIDTH / 2);
			Point startVertex2 = new Point(startPoint.getX() + JFAutomaton.MYWIDTH / 2, startPoint.getY() + JFAutomaton.MYWIDTH / 2);
			if (isBetweenPoints(startVertex1, point, startVertex2)) {
				return true;
			} else {
				Point endVertex1 = new Point(endPoint.getX() - JFAutomaton.MYWIDTH / 2, endPoint.getY() - JFAutomaton.MYWIDTH / 2);
				Point endVertex2 = new Point(endPoint.getX() + JFAutomaton.MYWIDTH / 2, endPoint.getY() + JFAutomaton.MYWIDTH / 2);
				if (isBetweenPoints(endVertex1, point, endVertex2)) {
					return false;
				} else {
					return null;
				}
			}
		} else {
			Point rigthVertexPoint = new Point(start.getX() - JFAutomaton.MYWIDTH, start.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2 - JFAutomaton.MYWIDTH / 2);
			Point leftVertexPoint = new Point(start.getX(),  start.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2 + JFAutomaton.MYWIDTH / 2); 
			if (isBetweenPoints(rigthVertexPoint, point, leftVertexPoint)) {
				return true;
			} else {
				rigthVertexPoint = new Point(start.getX(), start.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2 - JFAutomaton.MYWIDTH / 2);
				leftVertexPoint = new Point(start.getX() + JFAutomaton.MYWIDTH,  start.getY() - AutomatonDrawer.MYSTATEHEIGHT / 2 + JFAutomaton.MYWIDTH / 2);
				if (isBetweenPoints(rigthVertexPoint, point, leftVertexPoint)) {
					return false;
				} else {
					return null;
				}
			}
		}
	}
	
	public static IStatePresentation getNearStatePresentation(IAutomatonPresentation aPresentation, Point point) {
		for (IStatePresentation sPresentation : aPresentation.getStatesPresentations()) {
			Point point2 = new Point(sPresentation.getX(), sPresentation.getY());
			
			if (isInEllipse(point2, point, AutomatonDrawer.MYSTATEHEIGHT + 2 * JFAutomaton.MYWIDTH, AutomatonDrawer.MYSTATEWIDTH + 2 * JFAutomaton.MYWIDTH)) {
				return sPresentation;
			}
		}
		return null;
	}
	
	public static Set<IStatePresentation> makeStatePresentations(IAutomaton automaton) {
		Set<IStatePresentation> set = new LinkedHashSet<IStatePresentation>();
		int x = 40;
		int y = 40;
		int i = 0;
		for (IState state : automaton.getStates()) {
			if (i == 4) {
				i = 0;
				x = 40;
			//	y += 150;
			}
			set.add(new StatePresentation(state, x, y));
			x += 200;
			y += 50;
			i++;
		}
		return set;
	}

	
	public static IStatePresentation searchSideState(Point point, IAutomatonPresentation automatonPresentation) {
		for (IStatePresentation state : automatonPresentation.getStatesPresentations()) {
			Point startPoint = new Point(state.getX() + AutomatonDrawer.MYSTATEWIDTH / 2 - JFAutomaton.MYWIDTH / 2, state.getY()- JFAutomaton.MYWIDTH / 2);
			Point endPoint = new Point(state.getX() + AutomatonDrawer.MYSTATEWIDTH / 2 + JFAutomaton.MYWIDTH / 2, state.getY() + JFAutomaton.MYWIDTH / 2);
			if (isBetweenPoints(startPoint, point, endPoint)) {
				return state; 
			}
		}
		return null;
	}
}
