package ru.amse.bazylevich.faeditor.ui.interpreter;

import java.awt.FlowLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import ru.amse.bazylevich.faeditor.fautomaton.IState;
import ru.amse.bazylevich.faeditor.fautomaton.ITransition;
import ru.amse.bazylevich.faeditor.fautomaton.algorithms.interpreter.IInterpreter;
import ru.amse.bazylevich.faeditor.fautomaton.algorithms.interpreter.impl.Interpreter;
import ru.amse.bazylevich.faeditor.fautomaton.algorithms.interpreter.util.InvalidateAutomatonException;
import ru.amse.bazylevich.faeditor.fautomaton.algorithms.interpreter.util.Way;
import ru.amse.bazylevich.faeditor.ui.fautomaton.IStatePresentation;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.draw.impl.AutomatonWhithWayDrawer;

public class JInterpreter extends JComponent {
	
	private IInterpreter myInterpreter;
	private final JFAutomaton myJFAutomaton;
	private final List<ITransition> myTransitionsWay;
	private final List<IStatePresentation> myStatesWay;
	private int myStep;
	private String myString;
	private final AutomatonWhithWayDrawer myDrawAutomaton;
	private final JTextField myInterpreterTextField;
	private final JButton myNextButton;
	private final JButton myPreviousButton;
	private final JDrawString myDrawString;
	
	public JInterpreter(JFAutomaton jFAutomaton, AutomatonWhithWayDrawer draw, JDrawString drawString) {
		myJFAutomaton = jFAutomaton;
		myTransitionsWay = new LinkedList<ITransition>();
		myStatesWay = new LinkedList<IStatePresentation>();
		myStep = 0;
		myDrawAutomaton = draw;
		myDrawString = drawString;
		setLayout(new FlowLayout());
		add(new JLabel("String :"));
		
		myInterpreterTextField = new JTextField(15);
		myInterpreterTextField.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyCode() == 10) {
					setString(myInterpreterTextField.getText());
				} else {
					stop();
				}
			}
		});
		add(myInterpreterTextField);
		
		myPreviousButton = new JButton("Previous step");
		myPreviousButton.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				decreaseStep();
			}
		});
		add(myPreviousButton);
		
		myNextButton = new JButton("Start");
		myNextButton.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				increaseStep();
			}
		});
		add(myNextButton);
		
		setEnabledPreviousButton();
		
		JButton stopButton = new JButton("Stop");
		stopButton.addMouseListener( new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				stop();
			}
		});
		add(stopButton);
	}
	
		
	public void setString(String str) {
		myString = str;
	}
	

	public void start() {
		myInterpreter = new Interpreter(myJFAutomaton.getAutomatonPresentation().getAutomaton());
		myDrawAutomaton.clearLists();
		myJFAutomaton.clearSelectedStates();
		myJFAutomaton.setSelectedTransition(null);
		myJFAutomaton.repaint();
		
		setString(myInterpreterTextField.getText());
		if (myString != null) {
			Way way = null;
			try {
				way = myInterpreter.getWay(myString);
			} catch (InvalidateAutomatonException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), "Warning", JOptionPane.WARNING_MESSAGE);
				myString = null;
			}
			if (way != null) {
				myNextButton.setText("Next step");
				myDrawString.setIsAcceptedString(way.isAccepted());
				myTransitionsWay.addAll(way.getTransitionsWay());
				makeStatesPresentationsList(way.getStatesWay());
		
				myStep = 0;
				myDrawString.setString(myString);
				myDrawAutomaton.setString(myString);
				setEnabledPreviousButton();
				myDrawAutomaton.addStates(myStatesWay.get(myStep));
				myJFAutomaton.repaint();
			}
		
		}
		
	}
	
	public void increaseStep() {
		if (myString == null) {
			start();
		} else {
			if (myStatesWay.size() > myStep + 1) {
				myStep++;
				setEnabledButtons();
				myDrawString.incStep();
				doForwardStep();
				myJFAutomaton.repaint();
			}
		}
	}
	
	
	public void decreaseStep() {
		if (myString != null) {
			if (myStep > 0) {
				myStep--;
				setEnabledButtons();
				myDrawString.decStep();
				doBackwardStep();
				myJFAutomaton.repaint();
			}
		}
	}

	
	public void stop() {
		myNextButton.setText("Start");
		myStatesWay.clear();
		myTransitionsWay.clear();
		myDrawString.setString(null);
		myDrawAutomaton.setString(null);
		myString = null;
		myStep = 0;
		setEnabledPreviousButton();
		myNextButton.setEnabled(true);
		myDrawAutomaton.clearLists();
		myJFAutomaton.repaint();
	}

	
	private void makeStatesPresentationsList(List<IState> list) {
		for (IState state : list) {
			myStatesWay.add(myJFAutomaton.getAutomatonPresentation().getMapStatesPresentation().get(state));
		}
	}
	
	
	private void doForwardStep() {
		myDrawAutomaton.addTransition(myTransitionsWay.get(myStep - 1));
		myDrawAutomaton.addStates(myStatesWay.get(myStep));
	}
	
	private void doBackwardStep() {
		myDrawAutomaton.removeTransition();
		myDrawAutomaton.removeState();
	}
	
	private void setEnabledButtons() {
		setEnabledNextButton();
		setEnabledPreviousButton();
	}
	
	private void setEnabledPreviousButton() {
		if (myStep > 0) {
			myPreviousButton.setEnabled(true);
		} else {
			myPreviousButton.setEnabled(false);
		}
	}
	
	private void setEnabledNextButton() {
		if (myStep < myStatesWay.size() - 1) {
			myNextButton.setEnabled(true);
		} else {
			myNextButton.setEnabled(false);
		}
	}

}
