package ru.amse.bazylevich.faeditor.ui.menuactions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.bazylevich.faeditor.main.MainFrame;
import ru.amse.bazylevich.faeditor.ui.fautomaton.JFAutomaton;
import ru.amse.bazylevich.faeditor.ui.fautomaton.file.AutomatonPresentationSaver;
import ru.amse.bazylevich.faeditor.ui.fautomaton.file.util.Closer;

public class SaveAsAction  extends AbstractAction {
	
	protected final JFileChooser myFileChooser;
	protected final MainFrame myMainFrame;
	private final JFAutomaton myJFAutomaton;
	
	

	public SaveAsAction(String name, MainFrame mainFrame, JFAutomaton jAutomaton, JFileChooser fileChooser) {
		super(name);
		myFileChooser = fileChooser;
		myJFAutomaton = jAutomaton;
		myMainFrame = mainFrame;
	}

	
	@Override
	public void actionPerformed(ActionEvent e) {
		myFileChooser.setDialogTitle("Save");
		myFileChooser.setApproveButtonText("Save");
		int res = myFileChooser.showOpenDialog(myMainFrame);
		if ( res == JFileChooser.APPROVE_OPTION ) {
			save();
		} if (res == JFileChooser.CANCEL_OPTION) {
			myMainFrame.setAsked();
		}
	}
	
	protected void save() {
		Writer stream = null;
		XMLStreamWriter writer = null;
		File file = null;
		try {
			file  = myFileChooser.getSelectedFile();
			if (!file.getName().endsWith(".xml")) {
				file.delete();
				file = new File(file.getAbsolutePath() + ".xml");
			}
			stream = new FileWriter(file);
			XMLOutputFactory of = XMLOutputFactory.newInstance();
			writer = of.createXMLStreamWriter(stream);
			AutomatonPresentationSaver.saveToFile(writer, myJFAutomaton.getAutomatonPresentation());
			myMainFrame.setNormalTitle();
			if (myMainFrame.getCurrentFile() != null) {
				myMainFrame.setPreviosFile(myMainFrame.getCurrentFile());
			}
			myMainFrame.setCurrentFile(file);
		} catch (IOException exception) {
			JOptionPane.showMessageDialog(myMainFrame, "File can't be opened. Probably the program haven't access to it.","Warning" ,JOptionPane.WARNING_MESSAGE);
			processing();
		} catch (XMLStreamException exeption) {
			JOptionPane.showMessageDialog(myMainFrame, "File can't be written", "Warning" ,JOptionPane.WARNING_MESSAGE);
			processing();
		} finally {
			Closer.close(stream);
			Closer.close(writer);
		}
	}
	
	private void processing() {
		if (myMainFrame.getCurrentFile() != null) {
			myFileChooser.setSelectedFile(myMainFrame.getCurrentFile());
		} else if (myMainFrame.getPreviousFle() != null) {
			myFileChooser.setSelectedFile(myMainFrame.getPreviousFle());
		}
		if (myMainFrame.getCurrentFile() != null) {
			myMainFrame.setPreviosFile(myMainFrame.getCurrentFile());
		}
		myMainFrame.setCurrentFile(null);
		actionPerformed(null);
	}
	
}
